@@ -1,30 +1,14 @@
-import inspect
+"""
+Common testing utilities.
+"""
 from contextlib import contextmanager
 from functools import partial
 
 import eventlet
 from mock import Mock
+
 from nameko.containers import WorkerContextBase
 
-from nameko.dependencies import DependencyFactory, InjectionProvider
-
-
-def instance_factory(service_cls, **injections):
-    """ Return an instance of ``service_cls`` with its injected dependencies
-    replaced with Mock objects, or as given in ``injections``.
-    """
-    service = service_cls()
-    for name, attr in inspect.getmembers(service):
-        if isinstance(attr, DependencyFactory):
-            factory = attr
-            if issubclass(factory.dep_cls, InjectionProvider):
-                try:
-                    injection = injections[name]
-                except KeyError:
-                    injection = Mock()
-                setattr(service, name, injection)
-    return service
-
 
 def get_dependency(container, dependency_cls, **match_attrs):
     """ Inspect ``container.dependencies`` and return the first item that is
@@ -44,6 +28,15 @@ def get_dependency(container, dependency_cls, **match_attrs):
                 return dep
 
 
+def get_container(runner, service_cls):
+    """ Inspect ``runner.containers`` and return the first item that is
+    hosting an instance of ``service_cls``.
+    """
+    for container in runner.containers:
+        if container.service_cls == service_cls:
+            return container
+
+
 @contextmanager
 def wait_for_call(timeout, mock_method):
     """ Return a context manager that waits ``timeout`` seconds for
